﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class Enemigo : EntidadMapa {

    public ParticleSystem particulasImpacto;
    public ParticleSystem particulasExplosion;
    public NavMeshAgent navagent;   // JPS set public for debug 28/11/2017
    public Vector3 target;

	void Start () {
        
    }

    public void Update()
    {
        base.Update();
        if (target!=null && navagent!=null)
        {
            if (isValidVector3(target))
            {
                if (!navagent.destination.Equals(target))
                {
                    //PanelDebug.staticAddTextoDerecha(string.Format("target: [{0}] destination: [{1}]", target.ToString(), navagent.destination.ToString()));
                    navagent.destination = target;
                }
            }
        }
    }

    private bool isValidVector3(Vector3 p_vector)
    {
        return (isValidValue(p_vector.x) && isValidValue(p_vector.y) && isValidValue(p_vector.z));
    }

    private bool isValidValue(float p_value)
    {
        return (!float.IsInfinity(p_value) && !float.IsNegativeInfinity(p_value) && !float.IsPositiveInfinity(p_value) && !float.IsNaN(p_value));
    }

    /// <summary>
    /// Inflige daño en el enemigo
    /// </summary>
    /// <param name="p_dano">Cantidad de daño</param>
    /// <returns>Daño total recibido</returns>
    public float RecibirDano(float p_dano)
    {
        float dano = base.RecibirDano(p_dano);
        Debug.Log("Enemigo: " + p_dano.ToString());
        return dano;
    }

    /// <summary>
    /// Establece el objetivo del enemigo
    /// </summary>
    /// <param name="p_Punto">Lugar al que se dirige el enemigo</param>
    public void setTarget(Vector3 p_Punto)
    {
        if (navagent==null)
            navagent = GetComponent<NavMeshAgent>();
        if (p_Punto != null && navagent != null)
        {
            navagent.destination = new Vector3(p_Punto.x, transform.position.y, p_Punto.z);
            navagent.updatePosition = true;
            navagent.updateRotation = true;
            navagent.isStopped = false;
        }
    }

    /// <summary>
    /// Establece la velocidad del enemigo
    /// </summary>
    /// <param name="p_Velocidad"></param>
    public void setVelocidad(float p_Velocidad)
    {
        if (p_Velocidad > 0f && navagent != null)
            navagent.speed = p_Velocidad;
    }

    /// <summary>
    /// Establece la aceleración del enemigo
    /// </summary>
    /// <param name="p_Aceleracion"></param>
    public void setAceleracion(float p_Aceleracion)
    {
        if (p_Aceleracion > 0f && navagent != null)
            navagent.acceleration = p_Aceleracion;
    }

    /// <summary>
    /// Establece el nivel de dificultad del enemigo en "Fácil"
    /// </summary>
    public void setNivelFacil()
    {
        setVelocidad(0.05f);
        setAceleracion(6f);
    }

    /// <summary>
    /// Establece el nivel de dificultad del enemigo en "Normal"
    /// </summary>
    public void setNivelNormal()
    {
        setVelocidad(0.06f);
        setAceleracion(7.5f);
    }

    /// <summary>
    /// Establece el nivel de dificultad del enemigo en "Difícil"
    /// </summary>
    public void setNivelDificil()
    {
        setVelocidad(0.07f);
        setAceleracion(9f);
    }

    /// <summary>
    /// Establece el estado de pausa del enemigo
    /// </summary>
    /// <param name="p_EstadoPausa">Estado de pausa: true si se pone pausa, false si se reanuda</param>
    public void enPausa(bool p_EstadoPausa)
    {
        if (navagent != null)
        {
            if (p_EstadoPausa)
                navagent.isStopped = true;
            else
                navagent.isStopped = false;
        }
    }

    /// <summary>
    /// Destruye el enemigo
    /// </summary>
    public override void Destruir()
    {
        ParticleSystem particulas = Instantiate(particulasExplosion, transform.parent);
        particulas.transform.position = transform.position;
        particulas.Play();
        ManejadorDeSonido.ReproducirFxExplosion();
        Destroy(gameObject);
    }

    /// <summary>
    /// Comprueba colisión
    /// </summary>
    /// <param name="collision"></param>
    void OnCollisionEnter(Collision collision)
    {
        GameObject externalObject = collision.gameObject;
        if (externalObject.tag == "Capsula")
        {
            externalObject.GetComponent<Capsula>().RecibirDano(10f);
            Destruir();
        }

    }
}
