﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

public class Mapa {

    private static List<EntidadMapa> _entidades;

    static Mapa()
    {
        _entidades = new List<EntidadMapa>();
    }

    public static List<EntidadMapa> CargarArchivo(string p_NombreArchivo)
    {
        Debug.Log("Cargando mapa: " + p_NombreArchivo);
        _entidades.Clear();
        TextAsset txtAsset = Resources.Load("ARS_GameLevels/" + p_NombreArchivo) as TextAsset;
        string[] txtLineas = txtAsset.text.Split('\n');
        Debug.Log("Núm. de líneas: " + txtLineas.Length);
        foreach (string linea in txtLineas)
        {
            if (linea.Length > 0) { // evita líneas vacías
                // No procesa lineas de comentarios (comentarios comienzan por #)
                if (linea[0] != '#')
                {
                    //Debug.Log("linea sin comentario (" + linea.Length + " caracteres): " + linea);
                    string[] parametrosLinea = linea.Split(new string[] { ", " }, System.StringSplitOptions.None);
                    //Debug.Log("parametros linea: " + parametrosLinea.Length);
                    if (parametrosLinea.Length >= 5)
                    {
                        try
                        {
                            EEntidadesMapa entidad = (EEntidadesMapa)Enum.Parse(typeof(EEntidadesMapa), parametrosLinea[0].Trim());
                            Debug.Log("entidad correcta: " + entidad.ToString());

                            float posX;
                            float posY;
                            float posZ;
                            float rotY;

                            int poblacionMaxima;
                            int generacionesMaximas;
                            float tiempoRecarga;

                            switch (entidad)
                            {
                                case EEntidadesMapa.CAJA:
                                    if (parametrosLinea.Length == 5)
                                    {
                                        posX = parseOrDefault(parametrosLinea[1], 0f);
                                        posY = parseOrDefault(parametrosLinea[2], 0f);
                                        posZ = parseOrDefault(parametrosLinea[3], 0f);
                                        rotY = parseOrDefault(parametrosLinea[4], 0f);
                                        // Deja caer la caja para que se apoye en el objeto que haya debajo.
                                        // Evita que caiga al ponerla en la misma coordenada vertical que el suelo (0.0f)
                                        posY += 0.05f;
                                        ManejadorDeEscena3D.CrearCaja(new Vector3(posX, posY, posZ), rotY);
                                    }
                                    break;
                                case EEntidadesMapa.CAPSULA:
                                    if (parametrosLinea.Length == 5)
                                    {
                                        posX = parseOrDefault(parametrosLinea[1], 0f);
                                        posY = parseOrDefault(parametrosLinea[2], 0f);
                                        posZ = parseOrDefault(parametrosLinea[3], 0f);
                                        rotY = parseOrDefault(parametrosLinea[4], 0f);
                                        ManejadorDeEscena3D.CrearCapsula(new Vector3(posX, posY, posZ), rotY);
                                    }
                                    break;

                                case EEntidadesMapa.PUNTO_REGENERACION_ENEMIGOS:
                                    //#PUNTO_REGENERACION_ENEMIGOS, posicionX, posicionY, posicionZ, rotacionY, poblacionMaxima, generacionesMaximas, tiempoRecarga
                                    if (parametrosLinea.Length == 8)
                                    {
                                        posX = parseOrDefault(parametrosLinea[1], 0f);
                                        posY = parseOrDefault(parametrosLinea[2], 0f);
                                        posZ = parseOrDefault(parametrosLinea[3], 0f);
                                        rotY = parseOrDefault(parametrosLinea[4], 0f);
                                        poblacionMaxima = parseOrDefault(parametrosLinea[5], 1);
                                        generacionesMaximas = parseOrDefault(parametrosLinea[6], 1);
                                        tiempoRecarga = parseOrDefault(parametrosLinea[7], 1f);
                                        ManejadorDeEscena3D.CrearGeneradorDeEnemigos(new Vector3(posX, posY, posZ), rotY, poblacionMaxima, generacionesMaximas, tiempoRecarga);
                                    }
                                    break;
                            }
                        }
                        catch (Exception e)
                        {
                            Debug.Log("entidad incorrecta: " + parametrosLinea[0]);
                        }
                    }
                }
            }
        }
        return _entidades;
    }

    private static float parseOrDefault(string p_StringValue, float p_DefaultValue)
    {
        float value;
        if (float.TryParse(p_StringValue.Trim(), out value))
            return value;
        else
            return p_DefaultValue;
    }

    private static int parseOrDefault(string p_StringValue, int p_DefaultValue)
    {
        int value;
        if (int.TryParse(p_StringValue.Trim(), out value))
            return value;
        else
            return p_DefaultValue;
    }

}
