﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GamePlay : MonoBehaviour {

    public ContadorTiempo contadorTiempo;       // widget contador de tiempo (cuenta atrás)
    private static GamePlay reference;          // singleton

    public Camera cam;                          // cámara de la escena, se usa para saber a donde apunta el jugador
    public GameObject panelPausa;               // panel que contiene el menú "pausa"

    private bool isBotonIzquierdoPresionado;    // true mientras el botón izquierdo está presionado
    private bool isBotonDerechoPresionado;      // true mientras el botón derecho está presionado

    public ArmaYMunicion HUDArmaPrimaria;       // widget que muestra información del arma primaria
    public ArmaYMunicion HUDArmaSecundaria;     // widget que muestra información del arma secundaria

    void Start () {
        reference = this;                                       // singleton
        contadorTiempo.FinContador += OnFinContadorTiempo;      // captura evento de fin de contador

        panelPausa.transform.localScale = new Vector3(0, 0);    // ocultar menú pausa

        HUDArmaPrimaria.SetImagen(ArmaYMunicion.EIcono.pistola);
        HUDArmaSecundaria.SetImagen(ArmaYMunicion.EIcono.ametralladora);

        ManejadorDeEscena3D.enPausa(false);                     // inicia la partida
    }
	
	void Update () {
        if (reference.isBotonIzquierdoPresionado && Partida.armaSecundaria != null)
        {
            Partida.armaSecundaria.Disparar(reference.cam);
            if (!Partida.armaSecundaria.modoRafaga)
                reference.isBotonIzquierdoPresionado = false;
        }

        if (reference.isBotonDerechoPresionado && Partida.armaPrimaria != null)
        {
            Partida.armaPrimaria.Disparar(reference.cam);
            if (!Partida.armaPrimaria.modoRafaga)
                reference.isBotonDerechoPresionado = false;
        }

        if (reference.contadorTiempo.Valor >= 30 && Partida.powerupx2Generado == false)
        {
            float x = UnityEngine.Random.Range(-0.15f, 0.15f);
            float z = UnityEngine.Random.Range(-0.15f, 0.15f);
            ManejadorDeEscena3D.CrearPowerupX2(new Vector3(x, 2f, z));
            Partida.powerupx2Generado = true;
        }
    }

    /// <summary>
    /// La cuenta atrás ha finalizado
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void OnFinContadorTiempo(object sender, EventArgs e)
    {
        //PanelDebug.staticAddTextoDerecha("GamePlay script: Llega evento FinContador");
        SceneManager.LoadScene("FinDePartida");
    }

    public static void OnCapsulasDestruidas()
    {
        Partida.perdedor = true;
        SceneManager.LoadScene("FinDePartida");
    }

    /// <summary>
    /// Inicia la presión sobre el botón izquierdo
    /// </summary>
    public static void OnBotonIzquierdoDown()
    {
        //PanelDebug.staticAddTextoDerecha("boton izquierdo down");
        reference.isBotonIzquierdoPresionado = true;
    }

    /// <summary>
    /// Finaliza la presión sobre el botón izquierdo
    /// </summary>
    public static void OnBotonIzquierdoUp()
    {
        //PanelDebug.staticAddTextoDerecha("boton izquierdo up");
        reference.isBotonIzquierdoPresionado = false;
    }

    /// <summary>
    /// Inicia la presión sobre el botón derecho
    /// </summary>
    public static void OnBotonDerechoDown()
    {
        //PanelDebug.staticAddTextoDerecha("boton derecho down");
        reference.isBotonDerechoPresionado = true;
    }

    /// <summary>
    /// Finaliza la presión sobre el botón derecho
    /// </summary>
    public static void OnBotonDerechoUp()
    {
        //PanelDebug.staticAddTextoDerecha("boton derecho up");
        reference.isBotonDerechoPresionado = false;
    }

    /// <summary>
    /// Mostar menú pausa
    /// </summary>
    public void OnPausa()
    {
        reference.panelPausa.transform.localScale = new Vector3(1, 1);
        reference.panelPausa.transform.Find("menu").GetComponent<MenuPausa>().OnShow();
        ManejadorDeEscena3D.enPausa(true);
    }
}
