﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Scoreboard
{

    public class EntradaScoreboard
    {
        public string nombre;
        public int valor;

        public EntradaScoreboard(string p_Nombre, int p_Valor)
        {
            nombre = PrepararNombre(p_Nombre);
            valor = p_Valor;
        }

        private string PrepararNombre(string p_Nombre)
        {
            string nombrePreparado = p_Nombre.Trim();
            if (!string.IsNullOrEmpty(nombrePreparado))
                return nombrePreparado;
            else
                return "Jugador";
        }

        public override string ToString()
        {
            return nombre+":"+valor;
        }
    }

    private string path = Application.persistentDataPath + "/scoreboard.txt";

    private List<EntradaScoreboard> puntuaciones;

    private static Scoreboard singleton;

    private Scoreboard()
    {
        puntuaciones = new List<EntradaScoreboard>();
        Cargar();
    }

    public static Scoreboard getInstance()
    {
        if (singleton == null)
            singleton = new Scoreboard();
        return singleton;
    }

    private void Cargar()
    {
        string[] txtLineas;

        if (HandleTextFile.FileExists(path))
            txtLineas = HandleTextFile.ReadString(path);
        else
        {
            TextAsset txtAsset = Resources.Load("scoreboard") as TextAsset;
            txtLineas = txtAsset.text.Split('\n');
        }

        foreach (string linea in txtLineas)
        {
            string[] valoresLinea = linea.Split(':');
            if (valoresLinea.Length == 2)
            {
                int valor;
                if (int.TryParse(valoresLinea[1], out valor))
                {
                    puntuaciones.Add(new EntradaScoreboard(valoresLinea[0], valor));
                }
            }
        }
        puntuaciones.Sort((x, y) => (-1) * (x.valor.CompareTo(y.valor)));
    }

    public List<EntradaScoreboard> getPuntuaciones()
    {
        return puntuaciones;
    }


    public int VerificarNuevoRecord(int p_Puntuacion)
    {
        // devolver la posición en el ranking o 0 si no entra. Guardar el record.
        int posicion = 0;
        if (puntuaciones != null)
        {
            string nombreJugador = Preferencias.NombreJugador;
            if (puntuaciones.Count > 0)
            {
                int i = 0;
                bool encontrado = false;

                while (i < puntuaciones.Count && !encontrado)
                {
                    if (p_Puntuacion > puntuaciones[i].valor)
                    {
                        puntuaciones.Insert(i, new EntradaScoreboard(nombreJugador, p_Puntuacion));
                        encontrado = true;
                        posicion = i + 1;
                    }
                    else
                        i++;
                }
            }
            else
            {
                puntuaciones.Add(new EntradaScoreboard(nombreJugador, p_Puntuacion));
                posicion = 1;
            }

            if (posicion != 0)
                Save();

            return posicion;
        }
        else
        {
            Debug.Log("Scoreboard.VerificarNuevoRecord(" + p_Puntuacion.ToString() + "): puntuaciones = null");
            return 0;
        }
    }

    public void Save()
    {
        if (puntuaciones.Count>0)
        {
            List<string> top5OMenos = new List<string>();
            int i = 0;
            while (i<puntuaciones.Count && i<5)
            {
                top5OMenos.Add(puntuaciones[i].ToString());
                i++;
            }
            HandleTextFile.WriteString(path, top5OMenos.ToArray());
        }
    }

}
